package com.ruoyi.jjh.declaration.component;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.ent.entity.JContacts;
import com.ruoyi.jjh.ent.entity.JEnterpriseContact;
import com.ruoyi.jjh.ent.entity.JSmartDeclaration;
import com.ruoyi.jjh.ent.service.JContactsService;
import com.ruoyi.jjh.ent.service.JEnterpriseContactService;
import com.ruoyi.jjh.ent.service.JSmartDeclarationService;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import com.ruoyi.system.mapper.SysUserMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 企业或者政务自定义定时提醒组件
 *
 * @author du
 * @since 2024/7/18 9:55
 */
@Component("TimingRemind")
public class TimingRemindImpl implements TimingRemindService {

    private static final Logger log = LoggerFactory.getLogger(TimingRemindImpl.class);
    /**
     * 智能提醒关联表
     */
    @Resource
    private JSmartDeclarationService jSmartDeclarationService;
    /**
     * 企业信息
     */
    @Autowired
    private IBmsEnterpriseBasicInfoService bmsEnterpriseBasicInfoService;
    /**
     * 企业常用联系人
     */
    @Resource
    private JEnterpriseContactService jEnterpriseContactService;

    /**
     * 企业联系人 多个
     */
    @Resource
    private JContactsService jContactsService;

    /**
     * 是否是正式环境
     */
    @Value("${isTiming}")
    private Boolean isTiming;

    /**
     * 短信
     */
    @Resource
    private SmsAlertsService smsAlertsService;


    /**
     * 人员
     */
    @Resource
    private SysUserMapper sysUserMapper;

    /**
     * 企业端要定时新增数据，并且还要发短信
     */
    public void enterpriseTiming(String t, Long id) {
        List<JSmartDeclaration> listAdd = new ArrayList<>();
        //给所有的企业都要添加一条智能提醒数据
        //要给每个企业端对应的所有常用联系人发短信
        bmsEnterpriseBasicInfoService.list().forEach(x -> {
            JSmartDeclaration sd = new JSmartDeclaration();
            sd.setAlertTime(LocalDateTime.parse(t));
            sd.setIsRead(1);
            sd.setSmartRemindersId(id);
            sd.setTyshxydm(x.getTyshxydm());
            listAdd.add(sd);
            getContact(x.getTyshxydm(), "您有1个全局自定义通知,请前往系统首页查看");
        });
        jSmartDeclarationService.saveBatch(listAdd);
    }

    /**
     * 根据企业名称和企业信用代码获取该企业的常用联系人并且发送短信
     */
    public void getContact(String code, String s) {
        if (isTiming) {
            List<JEnterpriseContact> list1 = jEnterpriseContactService.lambdaQuery().eq(JEnterpriseContact::getEnterpriseCode, code).list();
            if (list1 != null && !list1.isEmpty()) {
                list1.forEach(y -> {
                    //先添加工单再发送短信
                    //可在导入或者新增的时候将该常用联系人的手机号添加进工单
                    smsAlertsService.enterSms(y.getEnterpriseName(), s, y.getContactPhone());
                    log.info(y.getEnterpriseName() + "发送短信成功!");
                });
            } else {
                List<JContacts> list = jContactsService.lambdaQuery().eq(JContacts::getTyshxydm, code)
                        .eq(JContacts::getDefaultcontact, "1").list();
                list.forEach(y -> {
                    //先添加工单再发送短信
                    smsAlertsService.addSsmPhone(y.getYddh());
                    //可在导入或者新增的时候将该常用联系人的手机号添加进工单
                    smsAlertsService.enterSms(y.getQymc(), s, y.getYddh());
                    log.info(y.getQymc() + "发送短信成功!");
                });
            }
        }
    }

    /**
     * 政务端要定时新增数据，并且还要发短信
     */
    public void chiefTiming(String t, Long id, String pid) {
        List<JSmartDeclaration> listAdd = new ArrayList<>();
        //给所有政务人员发通知和短信
        //添加用户进表的时候手机号已经经过处理
        SysUser user = new SysUser();
        user.setUserType("02");
        for (SysUser x : sysUserMapper.selectUserList(user)) {
            if ("0".equals(x.getDelFlag()) & "0".equals(x.getStatus())) {
                JSmartDeclaration sd = new JSmartDeclaration();
                sd.setIsRead(1);
                sd.setAlertTime(LocalDateTime.parse(t));
                sd.setSmartRemindersId(id);
                if ("null".equals(pid)) {
                    sd.setProjectId(null);
                } else {
                    sd.setProjectId(Long.valueOf(pid));
                }
                sd.setZwId(x.getUserId());
                if (isTiming) {
                    if (sd.getProjectId() != null & !x.getPhonenumber().isEmpty()) {
                        smsAlertsService.enterSms("政府人员-" + x.getNickName(), "您有1个项目自定义通知,请前往工作台查看," + "登录网址为:https://idp.sipac.gov.cn/bms/#/login", x.getPhonenumber());
                    }
                    if (!x.getPhonenumber().isEmpty() & sd.getProjectId() == null) {
                        smsAlertsService.enterSms("政府人员-" + x.getNickName(), "您有1个全局自定义通知,请前往工作台查看," + "登录网址为:https://idp.sipac.gov.cn/bms/#/login", x.getPhonenumber());
                    }
                }
                listAdd.add(sd);
            }
        }
        jSmartDeclarationService.saveBatch(listAdd);
    }
}
