package com.ruoyi.jjh.declaration.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.framework.config.ServerConfig;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.dto.ApprovalDeclarationRecordsQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsApprovalInfoUpdateDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsDeclarationRecordsDto;
import com.ruoyi.jjh.declaration.entity.vo.ApprovalDeclarationRecordsQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.BmsDeclarationRecordsQueryVo;
import com.ruoyi.jjh.declaration.mapper.BmsDeclarationRecordsMapper;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.List;

/**
 * 在线申报记录Controller
 *
 * @author farben
 * @date 2023-08-28
 */
@RestController
@RequestMapping("/system/declarationRecords")
@Api(tags = "在线申报记录")
public class BmsDeclarationRecordsController extends BaseController {
    @Resource
    private IBmsDeclarationRecordsService bmsDeclarationRecordsService;

    @Value("${ruoyi.profile}")
    private String fileAddress;


    @Value("${server.servlet.context-path}")
    private String path;


    @Resource
    private BmsDeclarationRecordsMapper bmsDeclarationRecordsMapper;

    @Autowired
    private ServerConfig serverConfig;

    /**
     * 审核列表
     */
    @ApiOperation(value = "审核列表")
    @PreAuthorize("@ss.hasAnyRoles('admin,gov,other-gov')")
    @GetMapping("/approvalList")
    public TableDataInfo approvalList(ApprovalDeclarationRecordsQueryDto approvalDeclarationRecordsQueryDto) {
        startPage();
        List<ApprovalDeclarationRecordsQueryVo> list = bmsDeclarationRecordsService.approvalList(approvalDeclarationRecordsQueryDto);
        return getDataTable(list);
    }

    @ApiOperation(value = "审批记录信息")
    @PreAuthorize("@ss.hasAnyRoles('admin,gov')")
    @Log(title = "审批记录信息", businessType = BusinessType.UPDATE)
    @PostMapping("/approvalEdit")
    public AjaxResult approvalEdit(@RequestBody BmsApprovalInfoUpdateDto bmsApprovalInfoUpdateDto) {
        return toAjax(bmsDeclarationRecordsService.updateApprovalInfo(bmsApprovalInfoUpdateDto));
    }

    /**
     * 查询在线申报记录列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent')")
    @ApiOperation(value = "查询在线申报记录列表")
    @GetMapping("/list")
    public TableDataInfo list(BmsDeclarationRecordsDto bmsDeclarationRecordsDto) {
        // 获取当前登录用户是否是企业端
        boolean admin = false;
        boolean b = false;
        String str = null;
        try {
            admin = SecurityUtils.isAdmin(SecurityUtils.getUserId());
            b = SecurityUtils.hasRole("ent");
            str = SecurityUtils.getUsername();
        } catch (Exception ignored) {
        }
        if (!admin && b) {
            bmsDeclarationRecordsDto.setCreditCode(str);
        }
        startPage();
        List<BmsDeclarationRecordsQueryVo> list = bmsDeclarationRecordsService.selectBmsDeclarationRecordsList(bmsDeclarationRecordsDto);
        return getDataTable(list);
    }

    /**
     * 根据id集合压缩所有文件
     */
//    @RequiresPermissions("system:declarationRecords:list")
    @GetMapping("/fileList")
    @ApiOperation("根据列表id集合压缩所有文件")
    public void fileList(@RequestParam(name = "list") List<Long> list, HttpServletResponse response) throws IOException {
        List<BmsDeclarationRecordsQueryVo> rd = bmsDeclarationRecordsMapper.selectDeclarationRecordsToIdList(list);
        String fatherFile = fileAddress + "/项目压缩文件";
        FileUtil.mkdir(fatherFile);
        for (BmsDeclarationRecordsQueryVo item : rd) {
            List<String> splitList = bmsDeclarationRecordsService.getAllFile(item.getTemplateJson());
            for (String s : splitList) {
                try {
                    String urls = RuoYiConfig.getProfile() + StringUtils.substringAfter(s, Constants.RESOURCE_PREFIX);
                    Path sourcePath = Paths.get(urls);
                    String itemsFileName = fatherFile + "/" + item.getEnterpriseName() + item.getProjectName() + "/" + sourcePath.getFileName();
                    FileUtil.touch(itemsFileName);
                    // 目标文件路径
                    Path targetPath = Paths.get(itemsFileName);
                    try {
                        Files.copy(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);
                        System.out.println("文件成功复制到新的位置！");
                    } catch (IOException e) {
                        throw new RuntimeException("暂无该文件!");
                    }
                } catch (Exception e) {
                    throw new RuntimeException("文件下载失败");
                }
            }
        }
        // 压缩后的文件路径
        String zipFilePath = fatherFile + ".zip";
        ZipUtil.zip(fatherFile, zipFilePath);
        File file = new File(zipFilePath);// 创建File对象
        response.setContentType("application/zip");// 设置内容类型为ZIP文件
        response.setHeader("Content-Disposition", "attachment; filename=\"file.zip\"");//设置文件名
        try (FileInputStream fileInputStream = new FileInputStream(file);
             OutputStream outputStream = response.getOutputStream()) {
            byte[] buffer = new byte[4096];
            int by;
            while ((by = fileInputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, by);
            }
        }
        FileUtil.del(zipFilePath);
        FileUtil.del(fatherFile);
    }

    /**
     * 查询在线申报记录列表-政务
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @GetMapping("/listRecords")
    public TableDataInfo listRecords(BmsDeclarationRecordsDto bmsDeclarationRecordsDto) {
        startPage();
        List<BmsDeclarationRecordsQueryVo> list = bmsDeclarationRecordsService.selectBmsDeclarationRecordsZwList(bmsDeclarationRecordsDto);
        return getDataTable(list);
    }


    /**
     * 导出在线申报记录列表
     */
//    @RequiresPermissions("system:declarationRecords:export")
    @Log(title = "在线申报记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmsDeclarationRecordsDto bmsDeclarationRecordsDto) {
        List<BmsDeclarationRecordsQueryVo> list = bmsDeclarationRecordsService.selectBmsDeclarationRecordsList(bmsDeclarationRecordsDto);
        ExcelUtil<BmsDeclarationRecordsQueryVo> util = new ExcelUtil<BmsDeclarationRecordsQueryVo>(BmsDeclarationRecordsQueryVo.class);
        util.exportExcel(response, list, "在线申报记录数据");
    }

    /**
     * 获取在线申报记录详细信息
     */
//    @RequiresPermissions("system:declarationRecords:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsDeclarationRecordsService.selectBmsDeclarationRecordsById(id));
    }

//    /**
//     * 新增在线申报记录
//     */
////    @RequiresPermissions("system:declarationRecords:add")
//    @Log(title = "在线申报记录", businessType = BusinessType.INSERT)
//    @PostMapping
//    public AjaxResult add(@RequestBody BmsDeclarationRecordsAddDto bmsDeclarationRecordsAddDto) {
//        return toAjax(bmsDeclarationRecordsService.insertBmsDeclarationRecords(bmsDeclarationRecordsAddDto));
//    }

    /**
     * 修改在线申报记录
     */
//    @RequiresPermissions("system:declarationRecords:edit")
    @Log(title = "在线申报记录", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody BmsDeclarationRecords bmsDeclarationRecords) {
        return toAjax(bmsDeclarationRecordsService.updateBmsDeclarationRecords(bmsDeclarationRecords));
    }


    /**
     * 删除在线申报记录
     */
//    @RequiresPermissions("system:declarationRecords:remove")
    @Log(title = "在线申报记录", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bmsDeclarationRecordsService.deleteBmsDeclarationRecordsByIds(ids));
    }
}
