package com.ruoyi.jjh.declaration.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.request.EnterpriseInfoRequest;
import com.ruoyi.jjh.declaration.entity.request.EnterpriseInfoWorkRequest;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.ent.service.JHeadQuartersRevenueService;
import com.ruoyi.jjh.ent.service.JServicesListService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 企业基础信息Controller
 *
 * @author farben
 * @date 2023-08-25
 */
@Api(tags = "企业基础信息")
@RestController
@RequestMapping("/system/enterpriseBasicInfo")
public class BmsEnterpriseBasicInfoController extends BaseController {

    @Autowired
    private IBmsEnterpriseBasicInfoService bmsEnterpriseBasicInfoService;

    @Resource
    private JServicesListService jServicesListService;

    @Resource
    private JHeadQuartersRevenueService jHeadQuartersRevenueService;

    /**
     * 查询企业基础信息列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,sightseer')")
    @ApiOperation(value = "查询企业基础信息列表", response = BmsEnterpriseBasicInfo.class)
    @GetMapping("/list")
    public AjaxResult list(Page<BmsEnterpriseBasicInfo> page, EnterpriseInfoRequest e) {
        return success(bmsEnterpriseBasicInfoService.page(page, e));
    }

    /**
     * 作业台查询企业信息
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "作业台查询企业信息", response = BmsEnterpriseBasicInfo.class)
    @GetMapping("/workSearch")
    public AjaxResult workSearch(EnterpriseInfoWorkRequest enterpriseInfoWorkRequest) {
        return AjaxResult.success(bmsEnterpriseBasicInfoService.workSearch(enterpriseInfoWorkRequest));
    }

    /**
     * 导出企业基础信息列表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,sightseer')")
    @ApiOperation(value = "导出企业基础信息列表")
    @Log(title = "企业基础信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EnterpriseInfoRequest e) {
        List<BmsEnterpriseBasicInfo> list = bmsEnterpriseBasicInfoService.page(e);
        ExcelUtil<BmsEnterpriseBasicInfo> util = new ExcelUtil<BmsEnterpriseBasicInfo>(BmsEnterpriseBasicInfo.class);
        util.exportExcel(response, list, "企业基础信息数据");
    }

    /**
     * 导入企业基础信息列表
     */
    @Transactional(rollbackFor = Exception.class)
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "导入企业基础信息列表")
    @Log(title = "导入企业基础信息列表", businessType = BusinessType.IMPORT)
    @PostMapping(value = "/importEnterprise", consumes = "multipart/form-data")
    public AjaxResult importEnterprise(@RequestPart("file") MultipartFile file, String type) throws Exception {
        List<BmsEnterpriseBasicInfo> allList = bmsEnterpriseBasicInfoService.list();
        ExcelUtil<BmsEnterpriseBasicInfo> util = new ExcelUtil<>(BmsEnterpriseBasicInfo.class);
        List<BmsEnterpriseBasicInfo> proList = util.importExcel(file.getInputStream());
        StringBuilder successMsg = new StringBuilder();
        if (proList == null || proList.isEmpty()) {
            throw new ServiceException("企业导入数据不能为空");
        } else {
            proList.forEach(entityA -> {
                entityA.setType(type != null ? type : "2");
                boolean b = allList.stream().anyMatch(entityB -> entityB.getTyshxydm().equals(entityA.getTyshxydm()));
                if (b) {
                    throw new ServiceException(entityA.getQymc() + "在企业库已重复,无法新增!");
                }
            });
            bmsEnterpriseBasicInfoService.saveBatch(proList);
            successMsg.append("导入成功");
        }
        return AjaxResult.success(successMsg);
    }

    /**
     * 根据信用代码获取该企业id
     */
    @ApiOperation(value = "根据信用代码获取该企业id", response = BmsEnterpriseBasicInfo.class)
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent,sightseer')")
    @GetMapping(value = "/getCreditCode")
    public AjaxResult getCreditCode(@RequestParam String getCreditCode) {
        // 获取当前登录用户是否是企业端
        boolean admin = false;
        boolean b = false;
        String str = null;
        try {
            admin = SecurityUtils.isAdmin(SecurityUtils.getUserId());
            b = SecurityUtils.hasRole("ent");
            str = SecurityUtils.getUsername();
        } catch (Exception ignored) {
        }
        if (!admin && b) {
            getCreditCode = str;
        }
        return success(bmsEnterpriseBasicInfoService.selectGetCreditCode(getCreditCode));
    }

    /**
     * 获取企业基础信息详细信息
     */
    @ApiOperation(value = "获取企业基础信息详细信息", response = BmsEnterpriseBasicInfo.class)
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent,sightseer')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsEnterpriseBasicInfoService.getById(id));
    }

    /**
     * 将企业库企业更改分类
     */
    @ApiOperation(value = "将企业库企业更改分类", response = BmsEnterpriseBasicInfo.class)
    @PreAuthorize("@ss.hasAnyRoles('admin')")
    @PostMapping(value = "/changeType", consumes = "multipart/form-data")
    public AjaxResult changeType(@RequestPart("file") MultipartFile file) throws IOException {
        List<BmsEnterpriseBasicInfo> list = bmsEnterpriseBasicInfoService.list();
        ExcelUtil<BmsEnterpriseBasicInfo> util = new ExcelUtil<>(BmsEnterpriseBasicInfo.class);
        List<BmsEnterpriseBasicInfo> proList = util.importExcel(file.getInputStream());

        //存储所有导入的信用代码
        Set<String> tyshxydmSet = new HashSet<>();
        for (BmsEnterpriseBasicInfo y : proList) {
            tyshxydmSet.add(y.getTyshxydm());
        }

        for (BmsEnterpriseBasicInfo x : list) {
            if (tyshxydmSet.contains(x.getTyshxydm())) {
                if(!"2".equals(x.getType())&& !"3".equals(x.getType())){
                    x.setType(x.getType()+","+"3");
                    bmsEnterpriseBasicInfoService.updateById(x);
                }
            }
        }

        for (BmsEnterpriseBasicInfo y : list) {
            proList.removeIf(x->x.getTyshxydm().equals(y.getTyshxydm()));
        }
        proList.forEach(x->{
            x.setType("3");
            bmsEnterpriseBasicInfoService.save(x);
        });
        return success();
    }
}
