package com.ruoyi.jjh.declaration.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseDirectory;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseDirectoryService;
import com.ruoyi.jjh.ent.entity.JContacts;
import com.ruoyi.jjh.ent.service.JContactsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 企业名录Controller
 *
 * @author farben
 * @date 2023-08-3100
 */
@Api(tags = "企业名录")
@RestController
@RequestMapping("/system/enterpriseDirectory")
public class BmsEnterpriseDirectoryController extends BaseController {
    @Autowired
    private IBmsEnterpriseDirectoryService bmsEnterpriseDirectoryService;

    @Resource
    private IBmsEnterpriseBasicInfoService iBmsEnterpriseBasicInfoService;

    @Resource
    private JContactsService jContactsService;

    /**
     * 查询企业名录列表
     */
//    @RequiresPermissions("system:enterpriseDirectory:list")
    @ApiOperation(value = "查询企业名录列表")
    @GetMapping("/list")
    public TableDataInfo list(BmsEnterpriseDirectory bmsEnterpriseDirectory) {
        startPage();
        List<BmsEnterpriseDirectory> list = bmsEnterpriseDirectoryService.selectBmsEnterpriseDirectoryList(bmsEnterpriseDirectory);
        return getDataTable(list);
    }

    /**
     * 导出企业名录列表
     */
//    @RequiresPermissions("system:enterpriseDirectory:export")
    @ApiOperation(value = "导出企业名录列表")
    @Log(title = "企业名录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody BmsEnterpriseDirectory bmsEnterpriseDirectory) {
        List<BmsEnterpriseDirectory> list = bmsEnterpriseDirectoryService.selectBmsEnterpriseDirectoryList(bmsEnterpriseDirectory);
        ExcelUtil<BmsEnterpriseDirectory> util = new ExcelUtil<BmsEnterpriseDirectory>(BmsEnterpriseDirectory.class);
        util.exportExcel(response, list, "企业名录数据");
    }

    /**
     * 获取企业名录详细信息
     */
//    @RequiresPermissions("system:enterpriseDirectory:query")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bmsEnterpriseDirectoryService.selectBmsEnterpriseDirectoryById(id));
    }

    /**
     * 新增企业名录
     */
//    @RequiresPermissions("system:enterpriseDirectory:add")
    @Log(title = "企业名录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BmsEnterpriseDirectory bmsEnterpriseDirectory) {
        return toAjax(bmsEnterpriseDirectoryService.insertBmsEnterpriseDirectory(bmsEnterpriseDirectory));
    }

    /**
     * 修改企业名录
     */
//    @RequiresPermissions("system:enterpriseDirectory:edit")
    @Log(title = "企业名录", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody BmsEnterpriseDirectory bmsEnterpriseDirectory) {
        return toAjax(bmsEnterpriseDirectoryService.updateBmsEnterpriseDirectory(bmsEnterpriseDirectory));
    }

    /**
     * 删除企业名录
     */
//    @RequiresPermissions("system:enterpriseDirectory:remove")
    @Log(title = "企业名录", businessType = BusinessType.DELETE)
    @PostMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(bmsEnterpriseDirectoryService.deleteBmsEnterpriseDirectoryByIds(ids));
    }

    /**
     * 导出用名字和企业库名字做对比企业库没有的企业
     */
//    @PreAuthorize("@ss.hasAnyRoles('admin')")
//    @ApiOperation(value = "导出用名字和企业库名字做对比企业库没有的企业")
//    @PostMapping(value = "/check", consumes = "multipart/form-data")
//    public void check(@RequestPart("file") MultipartFile file, HttpServletResponse response) throws Exception {
//        //利用企业名录的东西来获取所有企业
//        ExcelUtil<BmsEnterpriseDirectory> util = new ExcelUtil<>(BmsEnterpriseDirectory.class);
//        List<BmsEnterpriseDirectory> proList = util.importExcel(file.getInputStream());
//        //获取企业库的所有企业
//        List<BmsEnterpriseBasicInfo> list = iBmsEnterpriseBasicInfoService.list();
//        Set<String> bNames = list.stream()
//                .map(BmsEnterpriseBasicInfo::getQymc)
//                .collect(Collectors.toSet());
//        List<BmsEnterpriseDirectory> filteredList = proList.stream()
//                .filter(a -> !bNames.contains(a.getEnterpriseName()))
//                .collect(Collectors.toList());
////        System.out.println(filteredList.size());
//        //导出
//        util.exportExcel(response, filteredList, "企业名录数据");
//    }

    /**
     * 查询联系人
     */
//    @PreAuthorize("@ss.hasAnyRoles('admin')")
//    @ApiOperation(value = "查询联系人")
//    @PostMapping(value = "/searchContact", consumes = "multipart/form-data")
//    public AjaxResult searchContact(@RequestPart("file") MultipartFile file) throws Exception {
//        //利用企业名录的东西来获取所有企业
//        ExcelUtil<BmsEnterpriseDirectory> util = new ExcelUtil<>(BmsEnterpriseDirectory.class);
//        List<BmsEnterpriseDirectory> proList = util.importExcel(file.getInputStream());
//        //获取企业库的所有企业
//        List<BmsEnterpriseBasicInfo> list = iBmsEnterpriseBasicInfoService.list();
//        Set<String> bNames = proList.stream()
//                .map(BmsEnterpriseDirectory::getEnterpriseName)
//                .collect(Collectors.toSet());
//        List<BmsEnterpriseBasicInfo> filteredList = list.stream()
//                .filter(a -> bNames.contains(a.getQymc()))
//                .collect(Collectors.toList());
////        System.out.println(filteredList.size());
////        System.out.println(proList.size());
//        filteredList.forEach(x -> {
//            List<JContacts> list1 = jContactsService.lambdaQuery().eq(JContacts::getTyshxydm, x.getTyshxydm()).list();
//            if (!list1.isEmpty()) {
//                System.out.println(x.getQymc() + "----" + list1.size());
//            } else {
//                System.out.println(x.getQymc() + "----" + 0);
//            }
//        });
//        return success();
//    }

}
