package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.jjh.declaration.component.TimingRemindService;
import com.ruoyi.jjh.declaration.entity.BmsApprovalInfo;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsProcessInfo;
import com.ruoyi.jjh.declaration.entity.BmsTemplateRecord;
import com.ruoyi.jjh.declaration.entity.dto.ApprovalDeclarationRecordsQueryDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsApprovalInfoUpdateDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsDeclarationRecordsDto;
import com.ruoyi.jjh.declaration.entity.vo.ApprovalDeclarationRecordsQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.BmsDeclarationRecordsQueryVo;
import com.ruoyi.jjh.declaration.entity.vo.DeclarationRecordsVo;
import com.ruoyi.jjh.declaration.mapper.BmsDeclarationRecordsMapper;
import com.ruoyi.jjh.declaration.service.IBmsApprovalInfoService;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsProcessInfoService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import com.ruoyi.jjh.declaration.service.ICommonService;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JSmartDeclaration;
import com.ruoyi.jjh.ent.service.JProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 在线申报记录Service业务层处理
 *
 * @author farben
 * @date 2023-08-28
 */
@Service
public class BmsDeclarationRecordsServiceImpl extends ServiceImpl<BmsDeclarationRecordsMapper, BmsDeclarationRecords> implements IBmsDeclarationRecordsService {

    @Autowired
    private ICommonService iCommonService;


    @Resource
    private IBmsTemplateRecordService iBmsTemplateRecordService;


    @Autowired
    private IBmsApprovalInfoService iBmsApprovalInfoService;

    @Autowired
    private IBmsProcessInfoService iBmsProcessInfoService;

    @Autowired
    private IBmsEnterpriseBasicInfoService iBmsEnterpriseBasicInfoService;


    @Resource
    private JProjectService jProjectService;

    @Value("${ruoyi.profile}")
    private String fileAddress;

    /**
     * 短信
     */
    @Resource
    private TimingRemindService timingRemindImpl;

    /**
     * 查询在线申报记录
     *
     * @param id 在线申报记录主键
     * @return 在线申报记录
     */
    @Override
    public DeclarationRecordsVo selectBmsDeclarationRecordsById(Long id) {
        // 获取申请记录
        BmsDeclarationRecords declarationRecords = getById(id);
        DeclarationRecordsVo declarationRecordsVo = new DeclarationRecordsVo();
        declarationRecordsVo.setBmsDeclarationRecords(declarationRecords);
        // 在线模板记录id
        Long declarationTemplateType = declarationRecords.getTemplateId();
        //企业基本信息
        QueryWrapper<BmsEnterpriseBasicInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("tyshxydm", declarationRecords.getCreditCode());
        BmsEnterpriseBasicInfo one = iBmsEnterpriseBasicInfoService.getOne(queryWrapper);
        declarationRecordsVo.setBmsEnterpriseBasicInfo(one);
        declarationRecordsVo.setApprovalInfoList(iBmsApprovalInfoService.getApprovalInfoList(id));
        BmsTemplateRecord items = iBmsTemplateRecordService.getById(declarationRecords.getTemplateRecordId());
        //获取申报任务标题
        declarationRecordsVo.setEnterpriseDirectory(items.getEnterpriseDirectory());
        return declarationRecordsVo;
    }


    /**
     * 查询在线申报记录列表
     *
     * @param bmsDeclarationRecordsDto 在线申报记录
     * @return 在线申报记录
     */
    @Override
    public List<BmsDeclarationRecordsQueryVo> selectBmsDeclarationRecordsList(BmsDeclarationRecordsDto bmsDeclarationRecordsDto) {
        return baseMapper.selectBmsDeclarationRecordsList(bmsDeclarationRecordsDto);
    }

    @Override
    public List<BmsDeclarationRecordsQueryVo> selectBmsDeclarationRecordsZwList(BmsDeclarationRecordsDto bmsDeclarationRecordsDto) {
        return baseMapper.selectBmsDeclarationRecordsZwList(bmsDeclarationRecordsDto);
    }


    /**
     * 审核列表
     *
     * @param approvalDeclarationRecordsQueryDto
     * @return {@link List< ApprovalDeclarationRecordsQueryVo>}
     * @author emiya.xie
     * @create 2023/9/9 15:14
     */
    @Override
    public List<ApprovalDeclarationRecordsQueryVo> approvalList(ApprovalDeclarationRecordsQueryDto approvalDeclarationRecordsQueryDto) {
        if (SecurityUtils.getUserId().equals(1L)) {
            approvalDeclarationRecordsQueryDto.setApprovalDept(null);
        } else {
            approvalDeclarationRecordsQueryDto.setApprovalDept(SecurityUtils.getDeptId());
        }
        return baseMapper.approvalList(approvalDeclarationRecordsQueryDto);
    }

    /**
     * 审批
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateApprovalInfo(BmsApprovalInfoUpdateDto bmsApprovalInfoUpdateDto) {
        // 审批记录
        List<BmsApprovalInfo> list = iBmsApprovalInfoService.lambdaQuery().eq(BmsApprovalInfo::getDeclarationRecordsId, bmsApprovalInfoUpdateDto.getDeclarationRecordsId()).orderByDesc(BmsApprovalInfo::getSort).list();
        //最新一条审批记录
        BmsApprovalInfo ba = list.get(0);
        // 如果当前的deptId和最新一条的审批记录一样的话
        if (ba.getApprovalStatus() != 0 || !Objects.equals(ba.getApprovalDept(), SecurityUtils.getDeptId())) {
            throw new ServiceException("无法审批!");
        } else {
            ba.setId(ba.getId());
            ba.setApprovalById(SecurityUtils.getUserId());
            ba.setApprovalStatus(bmsApprovalInfoUpdateDto.getApprovalStatus());
            ba.setApprovalOpinions(bmsApprovalInfoUpdateDto.getApprovalOpinions());
            ba.setApprovalAttachment(bmsApprovalInfoUpdateDto.getApprovalAttachment());
            ba.setApprovalTime(DateTime.now());
            iBmsApprovalInfoService.updateById(ba);
        }
        int num = approval(bmsApprovalInfoUpdateDto, ba.getProcessId(), list.size() + 1);
        return num;
    }

    public int approval(BmsApprovalInfoUpdateDto bmsApprovalInfoUpdateDto, Long id, Integer a) {
        //获取审批之前的流程
        BmsProcessInfo bmsProcessInfo = iBmsProcessInfoService.getById(id);
        if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 1) {
            //根据审批之前的流程获取下一个流程id的详情数据
            if (bmsProcessInfo.getNextProcessId() != 0) {
                BmsProcessInfo newBmsProcessInfo = iBmsProcessInfoService.getById(bmsProcessInfo.getNextProcessId());
                BmsApprovalInfo ar = new BmsApprovalInfo();
                ar.setProcessId(bmsProcessInfo.getNextProcessId());
                ar.setDeclarationRecordsId(bmsApprovalInfoUpdateDto.getDeclarationRecordsId());
                ar.setApprovalStatus(0);
                ar.setApprovalDept(newBmsProcessInfo.getApprovalDept());
                ar.setSort(a + 1);
                ar.setIsReview(0);
                iBmsApprovalInfoService.save(ar);
            }
        }
        BmsDeclarationRecords bmsDeclarationRecords = baseMapper.selectById(bmsApprovalInfoUpdateDto.getDeclarationRecordsId());
        BmsEnterpriseBasicInfo one = iBmsEnterpriseBasicInfoService.lambdaQuery().eq(BmsEnterpriseBasicInfo::getTyshxydm, bmsDeclarationRecords.getCreditCode()).one();
        JProject jp = new JProject();
        jp.setId(bmsDeclarationRecords.getJjhProjectId());
        if (bmsProcessInfo.getApprovalLevel().equals(0)) {
            if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 1) {
                jp.setStatus(2);
                bmsDeclarationRecords.setStatus(2L);
            }
            if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 2) {
                jp.setStatus(8);
                bmsDeclarationRecords.setStatus(8L);
                timingRemindImpl.getContact(one.getTyshxydm(), "您有1个申报任务[初审不通过]");
            }
            if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 3) {
                //生成审批
                iCommonService.insertAuditRecord(bmsApprovalInfoUpdateDto.getDeclarationRecordsId(), 0);
                jp.setStatus(10);
                bmsDeclarationRecords.setStatus(10L);
                timingRemindImpl.getContact(one.getTyshxydm(), "您有1个申报任务[初审退回],待重新填报");
            }
        } else if (bmsProcessInfo.getApprovalLevel().equals(1)) {
            if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 1) {
                jp.setStatus(3);
                bmsDeclarationRecords.setStatus(3L);
            } else {
                jp.setStatus(7);
                bmsDeclarationRecords.setStatus(7L);
                timingRemindImpl.getContact(one.getTyshxydm(), "您有1个申报任务[复审不通过]");
            }
        } else if (bmsProcessInfo.getApprovalLevel().equals(2)) {
            if (bmsApprovalInfoUpdateDto.getApprovalStatus() == 1) {
                jp.setStatus(5);
                bmsDeclarationRecords.setStatus(5L);
                timingRemindImpl.getContact(one.getTyshxydm(), "您有1个申报任务[终审通过]");
            } else {
                jp.setStatus(9);
                bmsDeclarationRecords.setStatus(9L);
                timingRemindImpl.getContact(one.getTyshxydm(), "您有1个申报任务[终审不通过]");
            }
        }
        jProjectService.updateById(jp);
        return baseMapper.updateById(bmsDeclarationRecords);
    }


//    /**
//     * 新增在线申报记录
//     *
//     * @param bmsDeclarationRecordsAddDto 在线申报记录
//     * @return 结果
//     */
//    @Override
//    public int insertBmsDeclarationRecords(BmsDeclarationRecordsAddDto bmsDeclarationRecordsAddDto) {
//        iCommonService.checkEnterprise(bmsDeclarationRecordsAddDto.getEnterpriseId(), bmsDeclarationRecordsAddDto.getTemplateId());
//        BmsDeclarationRecords bmsDeclarationRecords = new BmsDeclarationRecords();
//        BeanUtil.copyProperties(bmsDeclarationRecordsAddDto, bmsDeclarationRecords);
//        bmsDeclarationRecords.setMatter("");
//        // 新增在线申请记录
//        int num = baseMapper.insert(bmsDeclarationRecords);
//        // 批量保存字段信息
//        List<BmsFieldInfoAddDto> fieldList = bmsDeclarationRecordsAddDto.getFieldList();
//        List<BmsFieldInfo> fieldInfoList = new ArrayList<>();
//        if (null != fieldList && fieldList.size() > 0) {
//            fieldList.forEach(x -> {
//                BmsFieldInfo bmsFieldInfo = new BmsFieldInfo();
//                BeanUtil.copyProperties(x, bmsFieldInfo);
//                bmsFieldInfo.setDetailId(bmsDeclarationRecords.getId());
//                fieldInfoList.add(bmsFieldInfo);
//            });
//            iBmsFieldInfoService.saveOrUpdateBatch(fieldInfoList);
//        }
//        // 批量保存文件信息
//        List<BmsFileInfoAddDto> fileList = bmsDeclarationRecordsAddDto.getFileList();
//        List<BmsFileInfo> fileInfoList = new ArrayList<>();
//        if (null != fileList && fileList.size() > 0) {
//            fileList.forEach(x -> {
//                BmsFileInfo bmsFileInfo = new BmsFileInfo();
//                BeanUtil.copyProperties(x, bmsFileInfo);
//                bmsFileInfo.setDetailId(bmsDeclarationRecords.getId());
//                fileInfoList.add(bmsFileInfo);
//            });
//            iBmsFileInfoService.saveOrUpdateBatch(fileInfoList);
//        }
//        return num;
//    }

    /**
     * 修改在线申报记录
     *
     * @param bmsDeclarationRecords 在线申报记录
     * @return 结果
     */
    @Override
    public int updateBmsDeclarationRecords(BmsDeclarationRecords bmsDeclarationRecords) {
        bmsDeclarationRecords.setUpdateTime(DateUtils.getNowDate());
        return baseMapper.updateBmsDeclarationRecords(bmsDeclarationRecords);
    }


    /**
     * 批量删除在线申报记录
     *
     * @param ids 需要删除的在线申报记录主键
     * @return 结果
     */
    @Override
    public int deleteBmsDeclarationRecordsByIds(Long[] ids) {
        return baseMapper.deleteBmsDeclarationRecordsByIds(ids);
    }

    /**
     * 删除在线申报记录信息
     *
     * @param id 在线申报记录主键
     * @return 结果
     */
    @Override
    public int deleteBmsDeclarationRecordsById(Long id) {
        return baseMapper.deleteBmsDeclarationRecordsById(id);
    }


    /**
     * 根据id集合压缩所有文件使用类
     */
    public List<String> getAllFile(String a) {
        JSONArray js1 = JSONObject.parse(a).getJSONArray("fields");
        List<String> strList = new ArrayList<>();
        for (Object o : js1) {
            JSONObject parse = JSONObject.parse(o.toString());
            if ("el-upload".equals(parse.get("tag"))) {
                if (parse.get("defaultValue") != null) {
                    String[] split = parse.get("defaultValue").toString().split(",");
                    strList.addAll(Arrays.asList(split));
                }
            }
        }
        return strList;
    }


    /**
     * 根据企业id和模板id查询
     *
     * @return
     */
    @Override
    public List<BmsDeclarationRecords> selectET(Long templateId, List<String> list) {
        return baseMapper.selectET(templateId, list);
    }


    /**
     * 申报模板不限时并且未填报的
     *
     * @return
     */
    @Override
    public List<JSmartDeclaration> allNoLimited() {
        return baseMapper.allNoLimited();
    }

    /**
     * 获取某个在线记录id可以添加到项目库的数据
     *
     * @return
     */
    @Override
    public JProject getAddProject(Long id) {
        return baseMapper.getAddProject(id);
    }


}
