package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.jjh.declaration.entity.*;
import com.ruoyi.jjh.declaration.entity.request.EnterpriseFileResquest;
import com.ruoyi.jjh.declaration.entity.request.EnterpriseFillResquest;
import com.ruoyi.jjh.declaration.service.*;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.service.JProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author du
 * @since 2024/7/24 13:53
 */
@Service
public class BmsEnterpriseFillServiceImpl implements IBmsEnterpriseFillService {
    @Autowired
    private IBmsApprovalInfoService iBmsApprovalInfoService;
    @Resource
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;

    @Autowired
    private IBmsEnterpriseBasicInfoService bmsEnterpriseBasicInfoService;

    @Autowired
    private IBmsEnterpriseDirectoryService bmsEnterpriseDirectoryService;

    @Autowired
    private IBmsTemplateRecordService iBmsTemplateRecordService;

    @Autowired
    private ICommonService iCommonService;

    @Autowired
    private JProjectService jProjectService;

    /**
     * 企业填报
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean add(EnterpriseFillResquest enterpriseFillResquest) {
        BmsEnterpriseBasicInfo basicInfo = bmsEnterpriseBasicInfoService.getOne(Wrappers.lambdaQuery(BmsEnterpriseBasicInfo.class).eq(BmsEnterpriseBasicInfo::getTyshxydm, enterpriseFillResquest.getCreditCode()));
        if (BeanUtil.isEmpty(basicInfo)) {
            throw new ServiceException("申请企业未入企业库！");
        }
        BmsEnterpriseDirectory enterpriseDirectory = bmsEnterpriseDirectoryService.getOne(Wrappers.lambdaQuery(BmsEnterpriseDirectory.class).eq(BmsEnterpriseDirectory::getCreditCode, enterpriseFillResquest.getCreditCode()).eq(BmsEnterpriseDirectory::getTemplateRecordId, enterpriseFillResquest.getTemplateRecordId()));
        if (BeanUtil.isEmpty(enterpriseDirectory)) {
            throw new ServiceException("申请企业不在企业名录中！");
        }
        // 申请是否过期或者该申请是不限时的
        // 如果该申报任务是不需要审批的则修改状态为已填报
        BmsTemplateRecord templateRecord = iBmsTemplateRecordService.getById(enterpriseFillResquest.getTemplateRecordId());
        if (templateRecord.getStatus() == 0) {
            throw new ServiceException("该申请还未到填报时间");
        } else if (templateRecord.getStatus() == 1) {
            throw new ServiceException("该申请已经过期");
        }
        BmsDeclarationRecords de = new BmsDeclarationRecords();
        de.setId(enterpriseFillResquest.getDeclarationId());
        de.setTemplateJson(enterpriseFillResquest.getTemplateJson());
        //如果该申报任务是不需要审批的则修改状态为已填报
        if (templateRecord.getIsApproval() == 0) {
            de.setStatus(6L);
        } else {
            de.setStatus(1L);
            //新增项目到项目库
            //获取某个在线记录id可以添加到项目库的数据
            JProject addProject = iBmsDeclarationRecordsService.getAddProject(enterpriseFillResquest.getDeclarationId());
            addProject.setStatus(1);
            Map<String, String> sm = parseJson(enterpriseFillResquest.getTemplateJson());
            addProject.setOtherJson(sm.get("str"));
            addProject.setFileJson(sm.get("fileStr"));
            jProjectService.save(addProject);
            de.setJjhProjectId(addProject.getId());
            //生成审批
            iCommonService.insertAuditRecord(enterpriseFillResquest.getDeclarationId(), 0);
        }
        return iBmsDeclarationRecordsService.updateById(de);
    }


    /**
     * 解析模板json
     */
    @Override
    public Map<String, String> parseJson(String json) {
        JSONArray js1 = JSONObject.parse(json).getJSONArray("fields");
        JSONArray ps1 = new JSONArray();
        JSONArray ps2 = new JSONArray();
        Map<String, String> obj1 = new HashMap<>();
        StringBuilder str = new StringBuilder("{");
        StringBuilder fileStr = new StringBuilder("{");
        for (Object o : js1) {
            JSONObject parse = JSONObject.parse(o.toString());
            if (!"el-upload".equals(parse.get("tag"))) {
                ps1.add(parse.toString());
            } else {
                ps2.add(parse.toString());
            }
        }
        for (Object o : ps1) {
            JSONObject parse = JSONObject.parse(o.toString());
            //解析下拉框和单选框
            if ("el-select".equals(parse.get("tag")) || "el-radio-group".equals(parse.get("tag"))) {
                JSONArray k = parse.getJSONArray("options");
                for (int j = 0; j < k.size(); j++) {
                    if (JSONObject.parse(k.getString(j)).get("value").equals(parse.get("defaultValue"))) {
                        str.append('"').append(parse.get("label").toString()).append('"').append(":")
                                .append('"').append(JSONObject.parse(k.getString(j)).get("label")).append('"');
                    }
                }
            } else {
                str.append('"').append(parse.get("label").toString()).append('"').append(":");
                if (parse.get("defaultValue") == null) {
                    str.append('"').append(" ").append('"');
                } else {
                    String result = parse.get("defaultValue").toString()
                            .replace("[", "")
                            .replace("]", "")
                            .replace(",", "至")
                            .replace("\"", "");
                    if ("true".equals(result)) {
                        result = "是";
                    }
                    str.append('"').append(result).append('"');
                }
            }
            str.append(",");
        }
        str.append("}");
        obj1.put("str", str.toString().replaceAll(",(?!.*,)", ""));
        //解析文件
        for (Object o : ps2) {
            JSONObject s1 = JSONObject.parse(o.toString());
            fileStr.append('"').append(s1.get("label").toString()).append('"').append(":")
                    .append('"').append(s1.get("defaultValue")).append('"').append(",");
        }
        fileStr.append("}");
        obj1.put("fileStr", fileStr.toString().replaceAll(",(?!.*,)", ""));
        return obj1;
    }

    /**
     * 初审驳回之后企业重新填报的接口
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void update(EnterpriseFillResquest enterpriseFillResquest) {
        //获取该条申报数据
        BmsDeclarationRecords dc = iBmsDeclarationRecordsService.getById(enterpriseFillResquest.getDeclarationId());
        if (dc.getStatus() != 10L) {
            throw new ServiceException("该申报未被初审驳回!");
        }
        dc.setTemplateJson(enterpriseFillResquest.getTemplateJson());
        dc.setStatus(1L);
        iBmsDeclarationRecordsService.updateById(dc);
        JProject addProject = new JProject();
        addProject.setStatus(1);
        addProject.setId(dc.getJjhProjectId());
        Map<String, String> sm = parseJson(enterpriseFillResquest.getTemplateJson());
        addProject.setOtherJson(sm.get("str"));
        addProject.setFileJson(sm.get("fileStr"));
        jProjectService.updateById(addProject);
    }

    /**
     * 上传文件
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadFile(EnterpriseFileResquest req) {
        //获取该条申报数据
        BmsDeclarationRecords ac = iBmsDeclarationRecordsService.getById(req.getDeclarationId());
        BmsDeclarationRecords newAc = new BmsDeclarationRecords();
        newAc.setId(req.getDeclarationId());
        newAc.setTemplateJson(req.getTemplateJson());
        iBmsDeclarationRecordsService.updateById(newAc);
        JProject addProject = new JProject();
        addProject.setId(ac.getJjhProjectId());
        Map<String, String> sm = parseJson(req.getTemplateJson());
        addProject.setFileJson(sm.get("fileStr"));
        jProjectService.updateById(addProject);
    }
}
