package com.ruoyi.jjh.ent.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.jjh.ent.entity.JMemorandum;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JProjectFund;
import com.ruoyi.jjh.ent.entity.request.JTop5FundRequest;
import com.ruoyi.jjh.ent.entity.response.JServicesDevelopResponse;
import com.ruoyi.jjh.ent.entity.response.ThreeYearsFundResponse;
import com.ruoyi.jjh.ent.service.JChiefWorkService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 政务端工作台控制层
 * @author du
 * @since 2024/8/28 10:36
 */
@RestController
@RequestMapping("/jjh/chiefWork")
@Api(tags = "政务端工作台 8.28")
public class JChiefWorkController extends BaseController {

    @Resource
    private JChiefWorkService jChiefWorkService;


    /**
     * 政务端-工作台 服务业发展总览
     * (数据更新后建议修改 两条数据 1个去年总数的，1个本年上一季度的)
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 服务业发展总览", response = JServicesDevelopResponse.class)
    @GetMapping("/servicesDevelop")
    public AjaxResult getServicesDevelop() {
        return success(jChiefWorkService.servicesDevelop());
    }

    /**
     * 政务端-工作台 近三年资金拨付情况
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 近三年资金拨付情况",response = ThreeYearsFundResponse.class)
    @GetMapping("/threeYearsFund/{years}")
    public AjaxResult getThreeYearsFund(@PathVariable String years) {
        return success(jChiefWorkService.getThreeYearsFund(years));
    }

    /**
     * 政务端-工作台 企业获得资金拨付排名TOP5
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 企业获得资金拨付排名TOP5",response = JProjectFund.class)
    @GetMapping("/top5Fund")
    public AjaxResult getTop5Fund(JTop5FundRequest a) {
        return success(jChiefWorkService.getTop5Fund(a));
    }

    /**
     * 政务端-工作台 当年已申报完成的项目
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 当年已申报完成的项目",response = JProject.class)
    @GetMapping("/currentYearFinish")
    public AjaxResult currentYearFinish() {
        return success(jChiefWorkService.currentYearFinish());
    }

    /**
     * 政务端-工作台 项目追踪情况
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 项目追踪情况",response = JProject.class)
    @GetMapping("/projectTrace")
    public AjaxResult projectTrace() {
        return success(jChiefWorkService.projectTrace());
    }

    /**
     * 政务端-工作台 备忘录更新情况
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 备忘录更新情况", response = JMemorandum.class)
    @GetMapping("/memorandum")
    public AjaxResult memorandum() {
        return success(jChiefWorkService.memorandum());
    }
}
