package com.ruoyi.jjh.ent.controller;


import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.regular.TimeChange;
import com.ruoyi.jjh.ent.entity.JContacts;
import com.ruoyi.jjh.ent.entity.JMemorandum;
import com.ruoyi.jjh.ent.entity.request.JMemorandumRequest;
import com.ruoyi.jjh.ent.service.JMemorandumService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 备忘录(JMemorandum)表控制层
 *
 * @author makejava
 * @since 2024-03-26 10:34:03
 */
@RestController
@RequestMapping("/jjh/jMemorandum")
@Api(tags = "备忘录表")
public class JMemorandumController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private JMemorandumService jMemorandumService;

    @Resource
    private TimeChange ts;

    @Value("${enterpriseUrl}")
    private String enterpriseUrl;

    /**
     * 分页查询所有数据
     *
     * @param page               分页对象
     * @param jMemorandumRequest 查询实体
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent,sightseer')")
    @ApiOperation(value = "分页查询所有数据", response = JMemorandum.class)
    @GetMapping
    public AjaxResult selectAll(Page<JMemorandum> page, JMemorandumRequest jMemorandumRequest) {
        return success(jMemorandumService.page(page, jMemorandumRequest));
    }
    @PreAuthorize("@ss.hasAnyRoles('admin')")
    @ApiOperation(value = "定时更新法人库-法人基本信息（统一社会信用代码）")
    @GetMapping("/getEnterpriseInfo")
    public AjaxResult selectAll2() {
        ts.changeEnterpriseInfo();
        return success();
    }

    @PreAuthorize("@ss.hasAnyRoles('admin')")
    @ApiOperation(value = "定时更新法人库-企业联络人员（统一社会信用代码）")
    @GetMapping("/getChangeContacts")
    public AjaxResult selectAll1() {
        ts.changeContacts();
        return AjaxResult.success();
    }

    @PreAuthorize("@ss.hasAnyRoles('admin')")
    @ApiOperation(value = "查看法人库-企业联络人员")
    @GetMapping("/getOneContacts/{tyshxydm}")
    public AjaxResult selectAll3(@PathVariable String tyshxydm) {
            String contacts = enterpriseUrl + "/645383/qyllrytyshxydm";
            JSONArray arr = null;
            HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
            headers.put("Content-Type", "application/json;charset=utf-8");
            //请求体
            JSONObject jsonObject = new JSONObject();//存放参数
            jsonObject.set("tyshxydm", tyshxydm);
            jsonObject.set("pageIndex", 0);
            HttpResponse response = HttpRequest.post(contacts)
                    .addHeaders(headers)
                    .body(jsonObject.toString())
                    .execute();
            if (response.getStatus() == 200) {
                JSONObject jsonObj = JSONUtil.parseObj(response.body());
                if ("200".equals(jsonObj.get("code"))) {
                    JSONObject reqData = JSONUtil.parseObj(jsonObj.get("data"));
                     arr = JSONUtil.parseArray(reqData.get("data"));
                }
            }
        return AjaxResult.success(arr);
    }

    /**
     * 根据关联项目库ID或者企业库ID查询该ID下的备忘录
     *
     * @param jMemorandumRequest 查询实体
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,sightseer')")
    @ApiOperation(value = "根据关联项目库ID查询该ID下的备忘录", response = JMemorandum.class)
    @GetMapping("/getIdToList")
    public AjaxResult selectIdToAll(JMemorandumRequest jMemorandumRequest) {
        if (jMemorandumRequest.getType() == null) {
            throw new ServiceException("分类不可为空");
        }
        if (jMemorandumRequest.getProjectId() == null) {
            throw new ServiceException("ID不可为空");
        }
        return success(jMemorandumService.selectIdToAll(jMemorandumRequest));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据", response = JMemorandum.class)
    @GetMapping("{id}")
    public AjaxResult selectOne(@PathVariable Serializable id) {
        return success(jMemorandumService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param jMemorandum 实体对象
     * @return 新增结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public AjaxResult insert(@RequestBody JMemorandum jMemorandum) {
        return success(jMemorandumService.save(jMemorandum));
    }

    /**
     * 导出备忘录表
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,sightseer')")
    @ApiOperation(value = "导出备忘录表")
    @Log(title = "导出备忘录表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, JMemorandumRequest jemorandum) {
        List<JMemorandum> list = jMemorandumService.selectIdToAll(jemorandum);
        ExcelUtil<JMemorandum> util = new ExcelUtil<>(JMemorandum.class);
        util.exportExcel(response, list, "备忘录");
    }

    /**
     * 修改数据
     *
     * @param jMemorandum 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "修改数据")
    @PostMapping("/edit")
    public AjaxResult update(@RequestBody JMemorandum jMemorandum) {
        return success(jMemorandumService.updateById(jMemorandum));
    }

    /**
     * 删除数据
     *
     * @param id 主键结合
     * @return 删除结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "删除数据")
    @PostMapping("/{id}")
    public AjaxResult delete(@PathVariable Long id) {
        return success(jMemorandumService.removeById(id));
    }
}

