package com.ruoyi.jjh.ent.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JProjectFund;
import com.ruoyi.jjh.ent.entity.response.FundStatisticsResponse;
import com.ruoyi.jjh.ent.entity.response.JProjectFundResponse;
import com.ruoyi.jjh.ent.service.JProjectFundService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * 项目资金表(j_project_fund)控制层
 * @author du
 * @since 2024/8/8 15:31
 */
@RestController
@RequestMapping("/jjh/jProjectFund")
@Api(tags = "项目资金表")
public class JProjectFundController extends BaseController {

    @Resource
    private JProjectFundService jProjectFundService;

    /**
     * 查询该项目id下的资金情况
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov,ent,sightseer')")
    @ApiOperation(value = "查询该项目id下的资金情况", response = JProjectFund.class)
    @GetMapping("/{id}")
    public AjaxResult selectFund(@PathVariable Long id) {
        return success(jProjectFundService.selectFund(id));
    }

    /**
     * 政务端-工作台 资金引导拨付情况统计
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 资金引导拨付情况统计", response = FundStatisticsResponse.class)
    @GetMapping("/fundStatistics")
    public AjaxResult fundStatistics() {
        return success(jProjectFundService.fundStatistics());
    }

    /**
     * 政务端-工作台 拨付资金企业申请情况TOP5
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "政务端-工作台 拨付资金企业申请情况TOP5", response = JProjectFund.class)
    @GetMapping("/appropriationTop5")
    public AjaxResult appropriationTop5() {
        return success(jProjectFundService.appropriationTop5());
    }

    /**
     * 企业端-首页 企业荣誉资质关联项目进展
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,ent')")
    @ApiOperation(value = "企业端-首页企业荣誉资质 关联项目进展", response = JProject.class)
    @GetMapping("/enterpriseProject")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "1企业荣誉资质 2关联项目进展", required = true),
    })
    public AjaxResult getEnterpriseProject(@RequestParam Integer type) {
        String creditCode = null;
        try {
            creditCode = SecurityUtils.getUsername();
        } catch (Exception ignored) {
        }
        return success(jProjectFundService.getEnterpriseProject(type,creditCode));
    }

    /**
     * 企业端-首页 往年专项资金申报
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,ent')")
    @ApiOperation(value = "企业端-首页 往年专项资金申报", response = JProjectFundResponse.class)
    @GetMapping("/usualFund")
    public AjaxResult usualFund() {
        String username = null;
        try {
            username = SecurityUtils.getUsername();
        } catch (Exception ignored) {
        }
        return success(jProjectFundService.usualFund(username));
    }

}
