package com.ruoyi.jjh.ent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.jjh.ent.entity.JMemorandum;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JProjectFund;
import com.ruoyi.jjh.ent.entity.request.JTop5FundRequest;
import com.ruoyi.jjh.ent.entity.response.JServicesDevelopResponse;
import com.ruoyi.jjh.ent.entity.response.ThreeYearsFundResponse;
import com.ruoyi.jjh.ent.mapper.JChiefWorkMapper;
import com.ruoyi.jjh.ent.service.JChiefWorkService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.List;

/**
 * 政务端工作台表业务处理层
 *
 * @author du
 * @since 2024/5/13 9:58
 */
@Service
public class JChiefWorkImpl implements JChiefWorkService {

    @Resource
    private JChiefWorkMapper jChiefWorkMapper;

    /**
     * 政务端-工作台 服务业发展总览
     */
    @Override
    public List<JServicesDevelopResponse> servicesDevelop() {
        return jChiefWorkMapper.servicesDevelop();
    }

    /**
     * 政务端-工作台 企业获得资金拨付排名TOP5
     */
    @Override
    public List<JProjectFund> getTop5Fund(JTop5FundRequest a) {
        List<JProjectFund> top5Fund = jChiefWorkMapper.getTop5Fund(a);
        if(a.getYearsChange() == 1){
            top5Fund.forEach(x-> x.setAppropriationTime(String.valueOf(LocalDate.now().getYear()-1)));
        }else {
            top5Fund.forEach(x-> x.setAppropriationTime(LocalDate.now().getYear() - 3 +"-"+
                    (LocalDate.now().getYear() - 1)));
        }
        return top5Fund;
    }

    /**
     * 政务端-工作台 当年已申报完成的项目
     */
    @Override
    public List<JProject>  currentYearFinish() {
        return jChiefWorkMapper.currentYearFinish();
    }
    /**
     * 政务端-工作台 项目追踪情况
     */
    @Override
    public List<JProject> projectTrace() {
        return jChiefWorkMapper.projectTrace();
    }

    /**
     * 政务端-工作台 备忘录更新情况
     */
    @Override
    public List<JMemorandum> memorandum() {
        List<JMemorandum> memorandum = jChiefWorkMapper.memorandum();
        memorandum.forEach(x->{
            if(x.getContent().length()>10){
                x.setContent(x.getContent().substring(0, 10)+"...");
            }
        });
        return memorandum;
    }

    /**
     * 政务端-工作台 近三年资金拨付情况
     */
    @Override
    public List<ThreeYearsFundResponse> getThreeYearsFund(String years) {
        return jChiefWorkMapper.getThreeYearsFund( years);
    }
}
