package com.ruoyi.jjh.ent.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.job.TaskException;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.ent.entity.JSmartReminders;
import com.ruoyi.jjh.ent.entity.request.JAddJobSmart;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersDq;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersNDq;
import com.ruoyi.jjh.ent.mapper.JSmartRemindersMapper;
import com.ruoyi.jjh.ent.service.JSmartRemindersService;
import com.ruoyi.quartz.domain.SysJob;
import com.ruoyi.quartz.service.ISysJobService;
import org.quartz.SchedulerException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 智能提醒（j_smart_reminders）表业务实现层
 *
 * @author du
 * @since 2024/7/1 13:33
 */
@Service
public class JSmartRemindersServiceImpl extends ServiceImpl<JSmartRemindersMapper, JSmartReminders> implements JSmartRemindersService {

    @Autowired
    private ISysJobService jobService;

    @Value("${isTiming}")
    private Boolean isTiming;

    @Resource
    private IBmsDeclarationRecordsService bmsDeclarationRecordsService;

    /**
     * 分页查询所有数据
     *
     * @param page            分页对象
     * @param jSmartReminders 查询实体
     * @return 所有数据
     */
    @Override
    public Page<JSmartReminders> page(Page<JSmartReminders> page, JSmartReminders jSmartReminders) {
        return baseMapper.page(page, jSmartReminders);
    }

    /**
     * 新增定期提醒数据
     *
     * @param jSmartRemindersDq 实体对象
     * @return 新增结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long add(JSmartRemindersDq jSmartRemindersDq) throws SchedulerException, TaskException {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(jSmartRemindersDq, jSmartReminders);
        if (jSmartReminders.getAlertManner() == 2) {
            throw new ServiceException("不可新增不定期提醒!");
        }
        LocalDateTime nt = LocalDateTime.now().plusMinutes(5);
        if(jSmartRemindersDq.getAlertTime().isBefore(nt)){
            throw new ServiceException("提醒时间必须在当前时间5分钟后!");
        }
        save(jSmartReminders);
        updateAndAdd(jSmartReminders, true);
        return jSmartReminders.getId();
    }

    /**
     * 生成cron表达式
     *
     * @return cron表达式
     */
    @Override
    public String generateCron(LocalDateTime localDateTime) {
        int second = 0; // 固定为0秒
        int minute = localDateTime.getMinute();
        int hour = localDateTime.getHour();
        int dayOfMonth = localDateTime.getDayOfMonth();
        int month = localDateTime.getMonthValue();
        String dayOfWeek = "?";
        // 构造 Cron 表达式
        return String.format("%d %d %d %d %d %s",
                second, minute, hour, dayOfMonth, month, dayOfWeek);
    }


    /**
     * 修改或者新增定时任务
     */
    @Override
    public void updateAndAdd(JSmartReminders jSmartReminders, Boolean flag) throws SchedulerException, TaskException {
        SysJob sysJob = new SysJob();
        sysJob.setConcurrent("1");//不允许并发执行
        sysJob.setCronExpression(generateCron(jSmartReminders.getAlertTime()));//生成cron表达式
        if (jSmartReminders.getAlertRecipients() == 1) {
            //如果是企业的定时提醒
            sysJob.setInvokeTarget("TimingRemind.enterpriseTiming(" + "'" + jSmartReminders.getAlertTime() + "'" + "," + jSmartReminders.getId() + "L" + ")");
        } else {
            //如果是政务端的定时提醒
            sysJob.setInvokeTarget("TimingRemind.chiefTiming(" + "'" + jSmartReminders.getAlertTime() + "'" + "," + jSmartReminders.getId() + "L," + "'" + jSmartReminders.getProjectId() + "'" + ")");
        }
        sysJob.setJobGroup("DEFAULT");
        sysJob.setJobName(jSmartReminders.getRulesName());
        sysJob.setMisfirePolicy("3");
        sysJob.setStatus("0");
        if (flag) {
            jobService.insertJob(sysJob);
            JAddJobSmart ja = new JAddJobSmart();
            ja.setSmartId(jSmartReminders.getId());
            ja.setJobId(sysJob.getJobId());
            baseMapper.addJobSmart(ja);
        } else {
            JAddJobSmart jAddJobSmart = baseMapper.selectJobSmart(jSmartReminders.getId());
            sysJob.setJobId(jAddJobSmart.getJobId());
            jobService.updateJob(sysJob);
        }
    }


    /**
     * 修改定期提醒数据
     *
     * @param jSmartRemindersDq 实体对象
     * @return 修改结果
     */
    @Override
    public Long updateDq(JSmartRemindersDq jSmartRemindersDq) throws SchedulerException, TaskException {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(jSmartRemindersDq, jSmartReminders);
        if (jSmartReminders.getAlertManner() == 2) {
            throw new ServiceException("参数错误");
        }
        updateById(jSmartReminders);
        updateAndAdd(jSmartReminders, false);
        return jSmartReminders.getId();
    }

    /**
     * 修改不定期提醒数据
     *
     * @param ndq 实体对象
     * @return 修改结果
     */
    @Override
    public Long updateNdq(JSmartRemindersNDq ndq) {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(ndq, jSmartReminders);
        if (jSmartReminders.getAlertManner() == 1) {
            throw new ServiceException("参数错误");
        }
        updateById(jSmartReminders);
        return ndq.getId();
    }

    /**
     * 通过主键查询单条智能提醒规则数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @Override
    public JSmartReminders selectOneSmart(Serializable id) {
        return baseMapper.selectOneSmart(id);
    }


//    /**
//     * 企业端首页智能提醒查询所有数据
//     *
//     * @return 所有数据
//     */
//    @Override
//    public List<SmartGetDeclaration> selectWorkAll() {

    //获取企业的不定时提醒
//        企业端
//        JSmartReminders qy = getById(1);
//        String userName = null;
//        try {
//            userName = SecurityUtils.getUsername();
//        } catch (Exception ignored) {
//        }
    //查询当前企业申报记录，对应的申报模板不是长期有效并且申报记录未填的，
//        List<SmartGetDeclaration> bmsDeclarationRecords = bmsDeclarationRecordsService.allNoLimited(userName);
//        //如果当前时间在该申报模板的结束时间前自定义的提前天数之前，就去掉
//        bmsDeclarationRecords.removeIf(x -> LocalDateTime.now().isBefore(x.getEndTime().minusDays(qy.getDaysAdvance())));
//        bmsDeclarationRecords.forEach(y->{
//            y.setDaysAdvance(qy.getDaysAdvance());
//            y.setAlertTime(y.getEndTime().minusDays(qy.getDaysAdvance()));
//            y.setAlertContent(qy.getAlertContent());
//        });
//        //获取企业的定时提醒
//        List<JSmartReminders> list = lambdaQuery().eq(JSmartReminders::getAlertRecipients, 1).eq(JSmartReminders::getAlertManner, 1).list();
//        list.forEach(x->{
//            if(LocalDateTime.now().isAfter(x.getAlertTime())){
//                SmartGetDeclaration sd = new SmartGetDeclaration();
//                sd.setAlertContent(x.getAlertContent());
//                sd.setAlertTime(x.getAlertTime());
//                bmsDeclarationRecords.add(sd);
//            }
//        });
//        bmsDeclarationRecords.sort(Comparator.comparing(entity -> Math.abs(ChronoUnit.MINUTES.between(LocalDateTime.now(), entity.getAlertTime()))));
//        return bmsDeclarationRecords;
//        return null;
//    }
}

